// TakePictureSDK.cpp : Defines the exported functions for the DLL application.
//this dll can be used to take pictures.

#include "stdafx.h"
#include "PictureTaker.h"
#include "SerialCom.h"
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>
#include <winbase.h>
#include <windows.h>
#include "SerialCom.h"

SerialCom ComDriver;


 


//Structure that defines the input to the thread function

struct ThreadInput{
	int numRecords;
	InitStruct* is;
	int error;
	PictureTaker* thisPointer;
};

int orig_baud;

//Global events, to switch context between the child thread and the parent, for function call backs-specifically if the 
//application uses MFC


HANDLE WINAPI hEventFail = CreateEvent(NULL,FALSE,FALSE,"Receive Fail");//event set when receive fails
HANDLE WINAPI hPositionUpdate = CreateEvent(NULL,FALSE,FALSE,"Position update");//event set for function call back- status update
HANDLE WINAPI hLastRecord = CreateEvent(NULL,FALSE,FALSE,"Last Record");//event set to indicate that receiving record is complete
HANDLE objectHandles[3];
int progressPosition = 0;



 PictureTaker::PictureTaker(void)
{
}


 const int bauds[]={19200,38400,57600,9600,115200};
 const char baudstrings[5][5] = 
{
	"\x1bK7\r", //19200
	"\x1bK8\r", //38400
	"\x1bK9\r", //57600
	"\x1bK6\r",	//9600
	"\x1bSZ\r", //115200
};




///<summary>
///This method receives the records from the serial port, computes the CheckSum and verifies 
///if the record is read correctly
///</summary>

int PictureTaker::ReceiveRecord(unsigned char *buffer, unsigned int *rec_nr, unsigned int *rec_length, unsigned int *checksum)
{
	unsigned char data[10];

	for(;;)			// Wait for the :
	{
		if (ComDriver.ReadDataFromCom(data,1)!=1)
			return -1;
		if (data[0]==':')
			break;
	}

	if (ComDriver.ReadDataFromCom(data,2)!=2)		// rec. nr
		return -2;
	*rec_nr = MakeHex(data,2);

	if (ComDriver.ReadDataFromCom(data,4)!=4)		// rec length
		return -4;
	*rec_length = MakeHex(data,4);

	if (ComDriver.ReadDataFromCom(buffer,*rec_length)!=*rec_length)	// data len
		return -5;

	if (ComDriver.ReadDataFromCom(data,2)!=2)		// checksum
		return -6;
	*checksum = MakeHex(data,2);

	if (ComDriver.ReadDataFromCom(data,1)!=1)		// terminating CR
		return -6;
	if (data[0]!=0x0D)
		return -6;

	ComDriver.WriteDataToCom("\x06",1);			// The record is OK, so acknowledge
	return 0;
}



///<summary>
///The method TakePicture method takes a struct for initializing the serial communication and 
///setting up the device. Creates a thread which does the serial receive work.
///</summary>


///<returns>


		///case -1:
		///	"No response on picture command"
		///case -2:
		///	"No rec_nr received"
		///case -3:
		///"No data length received"
		///case -4:
		///"No data received"
		///case -5:
		///"Checksum error"
		///case -6:
		///"No CR received"
		///case -7:
		///"Cannot create file or write to it"
		///case -8:
		///"No scanner detected"
		///case -9:
		///"Cannot open COM Port"
/// </returns>		


int PictureTaker::TakePicture(InitStruct initStruct)
{
	unsigned char command[1300];
	int nChars,x;
	unsigned int rec_nr;
	unsigned int rec_length;
	unsigned int checksum;
	int num_records;
	char pict_cmd[200];

	sprintf_s(pict_cmd,"\x1b((\r" "@OPTO,%4d,%4d,%4d,%4d,1,8,0,%3d,0,1,0,0#",initStruct.startx,initStruct.starty,initStruct.endx,initStruct.endy,initStruct.quality);

    if( ComDriver.OpenComPort(initStruct.port ) == false)
    {
		 return -9;
    }

	ComDriver.SetHandShake(SerialCom::EHandshakeOff);
	ComDriver.SetTimeouts(100,10,10);
    ComDriver.SetByteSize(8);
    ComDriver.SetStopBits(ONESTOPBIT);
    ComDriver.SetParity(NOPARITY);

	initStruct.updateProgress(true,"Initializing...");

	for(x=4;x>=0;x--)
	{
	    ComDriver.SetBaudrate(bauds[x]);
		ComDriver.ClearComm();
		ComDriver.WriteDataToCom("\x1bWC\r",4);

		nChars = ComDriver.ReadDataFromCom(command,1);
		if (nChars && (command[0] == 0x06))
			break;
	}
	if (x < 0)
	{
		return x;
	}

	orig_baud = x;

	if (x!=4)
	{
		ComDriver.WriteDataToCom("\x1BSZ\r",4);		// Set OPI/MDI to 115200
		ComDriver.ReadDataFromCom(command,1);		// Wait for ACK
		if (!nChars || (command[0] != 0x06))
		{
			return -8;
		}
		ComDriver.WriteDataToCom("\x1BZ2\r",4);		// Save settings (and activate baudrate)
		Sleep(100);
	    ComDriver.SetBaudrate(115200);
		ComDriver.ClearComm();
		ComDriver.ReadDataFromCom(command,1);		// Wait for ACK
		if (!nChars || (command[0] != 0x06))
		{
			return -8;
		}
		ComDriver.ClearComm();
	}
	ComDriver.WriteDataToCom("\x1bWD\r",4);
	Sleep(200);
	ComDriver.ClearComm();

	ComDriver.WriteDataToCom(pict_cmd,sizeof(pict_cmd));
	for(x=0;x<200;x++)
	{
		Sleep(10);
		if (ComDriver.GetUnreadedBytes()>50)
			break;
	}

	if ((x=ReceiveRecord(command, &rec_nr, &rec_length, &checksum)) != 0)
	{
		
		return x;
				
	}
	initStruct.updateProgress(true,"transfering ...");
	// We have the first record, now get the coordiantes and the width and hight
	char *context;
	char *token = strtok_s( (char *)command, "," , &context);
	for(x=0;x<10;x++)
	{
		if (token == NULL)
		{
			return -1;
		}
		token = strtok_s( NULL, ",", &context);
	}
	token[5]=0;
	num_records=atoi(token) - 1;

//Create a new thread to do the work of receiving records while parent thread just waits for updating the controls

	DWORD threadId;
	ThreadInput threadInput;
	threadInput.is = &initStruct;
	threadInput.numRecords = num_records;
	threadInput.error = 0;
	threadInput.thisPointer = this;

	HANDLE hThread = CreateThread(NULL,0,ThreadFunc,&threadInput,0,&threadId);

	if(hThread == NULL)
		return -7;

	
	DWORD dwEvents;

	hEventFail = OpenEvent(EVENT_ALL_ACCESS,true,"Receive Fail");
	hPositionUpdate = OpenEvent(EVENT_ALL_ACCESS,true,"Position update");
	hLastRecord = OpenEvent(EVENT_ALL_ACCESS,true,"Last Record");


	objectHandles[0] = hEventFail;
	objectHandles[1] = hPositionUpdate;
	objectHandles[2] = hLastRecord;


	for(;;)
	{
		dwEvents = WaitForMultipleObjects(3,objectHandles,FALSE,INFINITE);
		
		if(dwEvents == (WAIT_OBJECT_0 + 1)) //callback if record is received
		{
			initStruct.statusTracker(progressPosition);
		}
		else 
		{
			break;
		}
	}

	DWORD dwThread =  WaitForSingleObject(hThread,INFINITE);

	//Clean exit : close handles
	CloseHandle(hEventFail);
	CloseHandle(hLastRecord);
	CloseHandle(hPositionUpdate);
	CloseHandle(hThread);

	ComDriver.WriteDataToCom("\x17",1);
	initStruct.updateProgress(false,"");
	return threadInput.error;

}



int PictureTaker:: MakeHex(unsigned char *string, int length)
{
	int x;
	int ret=0;

	for(x=0;x<length;x++)
	{
		ret <<= 8;
		ret += string[x];
	}
	return ret;

}

///<summary>
/// For clean exiting
///</summary>

void PictureTaker:: Exit()
{

	ComDriver.WriteDataToCom("\x18",1);			// Cancel the transfer
	Sleep(100);
	ComDriver.WriteDataToCom(baudstrings[orig_baud],4);
	Sleep(250);
	ComDriver.WriteDataToCom("\x1bZ2\r",4);		// Save settings (and activate baudrate)
	Sleep(100);
	ComDriver.CloseComPort();
	
}



 DWORD WINAPI PictureTaker::ThreadFunc(LPVOID lpParam)
{
	unsigned int rec_nr;
	unsigned int rec_length;
	unsigned int checksum;
	int err, num_records;
	unsigned char command[1300];

	
	ThreadInput* input = (ThreadInput*)lpParam;
	InitStruct* initStruct = (InitStruct*)input->is;
	num_records = (int)input->numRecords;
	PictureTaker* takePicture= (PictureTaker*)input->thisPointer;
	
	
	//Create a file in the directory specified by the app

    DWORD dwBytesWritten = 0; 
	HANDLE hTempFile = CreateFile(initStruct->fileName, // file name 
                           GENERIC_READ | GENERIC_WRITE, // open r-w 
                           0,                    // do not share 
                           NULL,                 // default security 
                           CREATE_ALWAYS,        // overwrite existing
                           FILE_ATTRIBUTE_NORMAL,// normal file 
                           NULL);                // no template 

	

    if (hTempFile == INVALID_HANDLE_VALUE) 
    { 
		input->error = -7;
		return -7;
    } 

	for(int x=0;x<num_records;x++)
	{

		int position = (100*x)/num_records;
		if (position > 1)
		{
			progressPosition = position;
			SetEvent(hPositionUpdate);
		}

		if ((err = takePicture->ReceiveRecord(command, &rec_nr, &rec_length, &checksum)) != 0)
		{
				input->error = err;
				SetEvent(hEventFail);
				return err;
		}

		BOOL fWrite  = WriteFile(hTempFile,command,rec_length,&dwBytesWritten,NULL);
		if(fWrite == FALSE)
		{
			input->error = -7;
			SetEvent(hEventFail);
			return -7;
		}
	}

	CloseHandle (hTempFile);
	SetEvent(hLastRecord);
	return input->error;
		
}

 