

#ifdef SERIALCOM_EXPORTS
    #define SERIALCOM_API __declspec(dllexport)
#else
	#define SERIALCOM_API __declspec(dllimport)
    #pragma message("automatic link to SerialCom.lib")   // <== add this line
    #pragma comment(lib, "SerialCom.lib")                // <== add this line
#endif

#ifndef __SERIAL_COM_H__ 
#define __SERIAL_COM_H__

// Set default overlapped on for NON WINCE application
#ifndef SERCOM_OVERLAPPED
	#ifdef WINCE
		#define SERCOM_OVERLAPPED false		// No overlapped available in WINCE
	#else
		#define USE_SERCOM_OVERLAPPED
		#define SERCOM_OVERLAPPED true		// USE defualt overlapped in normal WIN32 environment
	#endif
#endif

 class SERIALCOM_API SerialCom
{
public:
	typedef enum _EHandShake
	{
		EHandshakeUnknown		= -1,	// Unknown
		EHandshakeOff			=  0,	// No handshaking
		EHandshakeHardware		=  1,	// Hardware handshaking (RTS/CTS)
		EHandshakeSoftware		=  2	// Software handshaking (XON/XOFF)
	} 
	EHandshake;
public:
	SerialCom(void);
	virtual ~SerialCom(void);

	virtual bool	OpenComPort( UINT nComport, bool bUseOverlapped = SERCOM_OVERLAPPED);
	virtual bool	OpenComPort( TCHAR *pszComport, bool bUseOverlapped = SERCOM_OVERLAPPED );
	void	CloseComPort( void );
	const bool	IsComOpen( void ) { return (m_hComHandle != NULL); }	
	HANDLE	GetComHandle( void ) { return m_hComHandle; }

	bool	SetMask( DWORD dwMask = EV_RXCHAR | EV_BREAK | EV_ERR );
	DWORD	GetMask( void );
	void	ClearComm( void );

	virtual bool SetBaudrate( DWORD dwBaudrate );
	bool	SetParity( BYTE bParity );
	bool	SetByteSize( BYTE bByteSize );
	bool	SetStopBits( BYTE bStopBits );

	DWORD	GetBaudrate( void );
	BYTE    GetParity( void );
	BYTE	GetByteSize( void );
	BYTE	GetStopBits( void );

	void	SetPendingTime( DWORD dwPendingTime );
	bool	SetRTS( BOOL bEnable );
	bool	SetDTR( BOOL bEnable );

	bool	SetHandShake( EHandshake handShake );

	bool	WriteDataToCom( LPCVOID lpBuf, DWORD dwLength );
	DWORD	ReadDataFromCom( LPVOID lpBuf, DWORD dwToRead );

	bool	WaitEvent( DWORD dwTimeout = INFINITE );

	bool	IsCTSHigh( void );
	bool	IsDSRHigh( void );
	bool	IsRingHigh( void );
	bool	IsRLSDHigh( void );

	bool	SetTimeouts( DWORD ReadIntTime = MAXDWORD, DWORD ReadTTMulti = 0UL, DWORD ReadTTConst = 0UL, DWORD WriteTTMulti = 0UL, DWORD WriteTTConst = 0UL );

	DWORD	GetUnreadedBytes( void );
	bool	SetSerialBuffers( DWORD dwInBuffer, DWORD dwOutBuffer );

	DWORD	GetLastError( void ) { return m_dwLastError; }

//	static const DWORD SUPPORTED_BAUDRATES[];

protected:
#ifdef USE_SERCOM_OVERLAPPED
	HANDLE	m_hOvEvent;
	DWORD	m_dwPendingTime;
#endif

	HANDLE	m_hComHandle;
	//TCHAR	m_szComName[ 24 ];

	DWORD	m_dwMasks;
	DWORD	m_dwEvents;

	DWORD	m_dwLastError;

// Automatically initialize the DCBlength when using the CDCB class
	class CDCB : public DCB
	{
	public:
		CDCB() { DCBlength = sizeof( DCB); }
	};
};


#endif
